/*******************************************************************************
* File Name: Counter_Sync_PM.c  
* Version 2.20
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "Counter_Sync.h"

static Counter_Sync_backupStruct Counter_Sync_backup;


/*******************************************************************************
* Function Name: Counter_Sync_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_Sync_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void Counter_Sync_SaveConfig(void) 
{
    #if (!Counter_Sync_UsingFixedFunction)
        /* Backup the UDB non-rentention registers for PSoC5A */
        #if (CY_PSOC5A)
            Counter_Sync_backup.CounterUdb = Counter_Sync_ReadCounter();
            Counter_Sync_backup.CounterPeriod = Counter_Sync_ReadPeriod();
            Counter_Sync_backup.CompareValue = Counter_Sync_ReadCompare();
            Counter_Sync_backup.InterruptMaskValue = Counter_Sync_STATUS_MASK;
        #endif /* (CY_PSOC5A) */
        
        #if (CY_PSOC3 || CY_PSOC5LP)
            Counter_Sync_backup.CounterUdb = Counter_Sync_ReadCounter();
            Counter_Sync_backup.InterruptMaskValue = Counter_Sync_STATUS_MASK;
        #endif /* (CY_PSOC3 || CY_PSOC5LP) */
        
        #if(!Counter_Sync_ControlRegRemoved)
            Counter_Sync_backup.CounterControlRegister = Counter_Sync_ReadControlRegister();
        #endif /* (!Counter_Sync_ControlRegRemoved) */
    #endif /* (!Counter_Sync_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: Counter_Sync_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_Sync_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void Counter_Sync_RestoreConfig(void) 
{      
    #if (!Counter_Sync_UsingFixedFunction)     
        /* Restore the UDB non-rentention registers for PSoC5A */
        #if (CY_PSOC5A)
            /* Interrupt State Backup for Critical Region*/
            uint8 Counter_Sync_interruptState;
        
            Counter_Sync_WriteCounter(Counter_Sync_backup.CounterUdb);
            Counter_Sync_WritePeriod(Counter_Sync_backup.CounterPeriod);
            Counter_Sync_WriteCompare(Counter_Sync_backup.CompareValue);
            /* Enter Critical Region*/
            Counter_Sync_interruptState = CyEnterCriticalSection();
        
            Counter_Sync_STATUS_AUX_CTRL |= Counter_Sync_STATUS_ACTL_INT_EN_MASK;
            /* Exit Critical Region*/
            CyExitCriticalSection(Counter_Sync_interruptState);
            Counter_Sync_STATUS_MASK = Counter_Sync_backup.InterruptMaskValue;
        #endif  /* (CY_PSOC5A) */
        
        #if (CY_PSOC3 || CY_PSOC5LP)
            Counter_Sync_WriteCounter(Counter_Sync_backup.CounterUdb);
            Counter_Sync_STATUS_MASK = Counter_Sync_backup.InterruptMaskValue;
        #endif /* (CY_PSOC3 || CY_PSOC5LP) */
        
        #if(!Counter_Sync_ControlRegRemoved)
            Counter_Sync_WriteControlRegister(Counter_Sync_backup.CounterControlRegister);
        #endif /* (!Counter_Sync_ControlRegRemoved) */
    #endif /* (!Counter_Sync_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: Counter_Sync_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_Sync_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void Counter_Sync_Sleep(void) 
{
    #if(!Counter_Sync_ControlRegRemoved)
        /* Save Counter's enable state */
        if(Counter_Sync_CTRL_ENABLE == (Counter_Sync_CONTROL & Counter_Sync_CTRL_ENABLE))
        {
            /* Counter is enabled */
            Counter_Sync_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            Counter_Sync_backup.CounterEnableState = 0u;
        }
    #endif /* (!Counter_Sync_ControlRegRemoved) */
    Counter_Sync_Stop();
    Counter_Sync_SaveConfig();
}


/*******************************************************************************
* Function Name: Counter_Sync_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Counter_Sync_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void Counter_Sync_Wakeup(void) 
{
    Counter_Sync_RestoreConfig();
    #if(!Counter_Sync_ControlRegRemoved)
        if(Counter_Sync_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            Counter_Sync_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!Counter_Sync_ControlRegRemoved) */
    
}


/* [] END OF FILE */
