/**
This code is based on the cypress example AN52705 - DMA usage for the PSoC5.
For the original copyright, see the comment below.

 * Copyright Hendrik Lipka, 2011
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF Hendrik Lipka.
*/
/* --------------------------------------------------------------------------
* Copyright 2010, Cypress Semiconductor Corporation.
*
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international
* treaty provisions. Cypress hereby grants to licensee a personal,
* non-exclusive, non-transferable license to copy, use, modify, create
* derivative works of, and compile the Cypress Source Code and derivative
* works for the sole purpose of creating custom software in support of
* licensee product to be used only in conjunction with a Cypress integrated
* circuit as specified in the applicable agreement. Any reproduction,
* modification, translation, compilation, or representation of this
* software except as specified above is prohibited without the express
* written permission of Cypress.
* 
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED,
* WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising
* out of the application or use of any product or circuit described herein.
* Cypress does not authorize its products for use as critical components in
* life-support systems where a malfunction or failure may reasonably be
* expected to result in significant injury to the user. The inclusion of
* Cypress' product in a life-support systems application implies that the
* manufacturer assumes all risk of such use and in doing so indemnifies
* Cypress against all charges.
* 
* Use may be limited by and subject to the applicable Cypress software
* license agreement.
* -----------------------------------------------------------------------------
* Copyright (c) Cypress MicroSystems 2000-2003. All Rights Reserved.
*
*****************************************************************************
*  Project Name: Mem2Peripheral 
*  Device Tested: CY8C3866AXI, CY8C5588AXI
*  Software Version: PSoC Creator Beta 5
*  Complier tested: Keil(C51) and GCC
*  Related Hardware: CY8CKIT-001
*****************************************************************************
*/

#include <device.h>
#include <stdbool.h>

#include "dds.h"

// number of samples
#define TABLE_SIZE 32

// start frequency fpr user input
#define START_FREQ 10000

// minimum frequency
#define MIN_FREQ 10

// number of channels for VCO control (incl. potentiometer)
#define NO_FREQ_CHANNELS 3

// bit size of VCO ADC
#define VCO_BITS 15

// offset (in counts) for VCO input - should be 2^(VCO_BITS-2)
// (2^VCO_BITS is full range, only the half is used for single ended input, and its half should be the offset)
#define VCO_OFFSET 8192

/* Macro to store the initail value of VDAC */
#define VDAC_INITIAL_VALUE 0x00

/* The table that stores the samples for the sine wave.*/
CYCODE const uint8 sineTable[TABLE_SIZE] =
{
  127, 151, 175, 197, 216, 232, 244, 251,
  255, 251, 244, 232, 216, 197, 175, 151,
  128, 103, 79,  57,  38,  22,  10,  3, 
  0,   3,   10,  22,  38,  57,  79,  103
};

/* The table that stores the samples for the ramp wave. */
CYCODE const uint8 rampTable[TABLE_SIZE]=
{
  128, 136, 144, 152, 160, 168, 176, 184,
  192, 200, 208, 216, 224, 232, 240, 255,
  0,   8,   16,  24,  32,  40,  48,  56, 
  64,  72,  80,  88,  96,  104, 112, 120
};

#define SINE 0
#define RAMP 1

uint8 selectMode=0;
uint8 waveForm=SINE;

int32 userFreq=START_FREQ;

/* Variable declarations for DMA_SineWave */
uint8 DMA_Wave_Chan=0xff;
uint8 DMA_Wave_TD[1]={0xff};

/* DMA Configuration for DMA_SineWave */
#define DMA_Wave_BYTES_PER_BURST 1
#define DMA_Wave_REQUEST_PER_BURST 1

#define DMA_Wave_SRC_BASE (CYDEV_FLASH_BASE)
#define DMA_Wave_DST_BASE (CYDEV_PERIPH_BASE)

/* Variable declarations for DMA_ADC */
/* Move these variable declarations to the top of the function */
uint8 DMA_ADC_Chan;
uint8 DMA_ADC_TD[1];

/* DMA Configuration for DMA_ADC */
#define DMA_ADC_BYTES_PER_BURST 2
#define DMA_ADC_REQUEST_PER_BURST 1
#define DMA_ADC_SRC_BASE (CYDEV_PERIPH_BASE)
#define DMA_ADC_DST_BASE (CYDEV_SRAM_BASE)

int16 ADC_Freq_Samples[3];
uint8 currentChannel=0;

uint8* table;

void init_dds()
{
	// start dds wave output
	VDAC8_Wave_Start();
	
	Timer_WaveCtrl_Start();
	setFrequency(userFreq);
	selectDMASource(waveForm);

	// enable sync sub system
	isr_sync_Start();
	Counter_Sync_Start();
	Clock_Sync_Start();
	Comp_Sync_Start();

	// enable subsystems for VCO functionality
	AMuxSeq_Start();
	AMuxSeq_Next();
	
	ADC_Freq_Start();
	ADC_Freq_IRQ_Enable();
	ADC_Freq_StartConvert();

}

void adcIrq()
{
    ADC_Freq_StopConvert(); 
	AMuxSeq_Next();
	
	float f=(float)((int32)ADC_Freq_GetResult32());
	f=(f/1.1)+100;
	
	ADC_Freq_Samples[currentChannel]=(int16)f;
	
	currentChannel=(currentChannel+1)%3;
	
	ADC_Freq_StartConvert();
}

void resetDMA()
{
	if (0xff!=DMA_Wave_Chan)
	{
		CyDmaChDisable(DMA_Wave_Chan);
		DMA_Wave_DmaRelease();
		CyDmaTdFree(DMA_Wave_TD[0]);
	}
	/* Configuring the DMA channel for Sine wave generation */
	/*************************************************************************************************************/
    /* Step 1 */
    /* DMA Initializations done for both the DMA Channels
	 * Burst count = 1, (8 bit data transferred to VDAC one at a time),
	 * Request per bit = 1 (this will cause transfer of the bytes only with every new request)
	 * High byte of source address = Upper 16 bits of Flash Base address
     * High byte of destination address =  Upper bytes of the VDAC8 data register
    */
    DMA_Wave_Chan = DMA_Wave_DmaInitialize(DMA_Wave_BYTES_PER_BURST, DMA_Wave_REQUEST_PER_BURST, 
                                                   HI16(DMA_Wave_SRC_BASE), HI16(DMA_Wave_DST_BASE));
                                                   
    /* Step 2 */
	/* Allocate TD for DMA_SineWave Channel*/
    DMA_Wave_TD[0] = CyDmaTdAllocate();
    
    /* Check if the allocated TD is valid */
	if (DMA_Wave_TD[0] == DMA_INVALID_TD)
	{
		LCD_Position(0,0);
		LCD_PrintString("CANNOT PROCEED");
		
        while(1)
        {
            /* Wait in an indefinite loop as the initialization could not be completed */
            ;
        }
	}
    
    /* Step 3 */
    /* Configure the sine_TD
	 * Transfer count = SINETABLE_SIZE (number of bytes to transfer for a Sine Wave)
	 * Next Td = sine_TD (This example loops back to the same TD. This is an example of Auto repeat DMA
	 * Configuration = The source address should be incremented for every byte of transfer
	 *				   to transfer all the sample for the Sine Wave 
	*/
    CyDmaTdSetConfiguration(DMA_Wave_TD[0], TABLE_SIZE, DMA_Wave_TD[0], TD_INC_SRC_ADR);
    
    /* Step 4 */
    /* Configure the sine_TD for address
	 * Source address = Lower 16 bits of Sine table array
	 * Destination address = Lower 16 bits of VDAC8_SineWave register */
    CyDmaTdSetAddress(DMA_Wave_TD[0], LO16((uint32)table), LO16((uint32)VDAC8_Wave_Data_PTR));
    
    /* Step 5 */
	/* Map the TD to the DMA Channel */
    CyDmaChSetInitialTd(DMA_Wave_Chan, DMA_Wave_TD[0]);

	// start DMA and therefore the generator
	CyDmaChEnable(DMA_Wave_Chan, 1);
}

void syncIrq()
{
	resetDMA();
}

void setFrequency(uint32 freq)
{
	Timer_WaveCtrl_WritePeriod((uint16)(30000000/(freq*32))-1);
}

void selectDMASource(uint8 source)
{
	
	if (0==waveForm)
	{
		table=(uint8*)sineTable;
	}
	else
	if (1==waveForm)
	{
		table=(uint8*)rampTable;
	}
	
	resetDMA();
}


void do_ui_waveform(bool top_pressed, bool bottom_pressed, uint8 slider, bool init)
{
	if (init)
	{
		LCD_ClearDisplay();
		LCD_IsReady();
	
		LCD_PrintString("    Waveform");
		LCD_Position(1,3);
		LCD_PutChar(0);
		LCD_IsReady();
		LCD_Position(1,4);
		if (0==waveForm)
		{
			LCD_PrintString("Sine");
		}
		else
		if (1==waveForm)
		{
			LCD_PrintString("Ramp");
		}
		LCD_IsReady();
	}	
	
	// has the wave form changed?
	if (top_pressed)
	{
		waveForm=(waveForm+1)%2;
		selectDMASource(waveForm);
		LCD_Position(1,4);
		if (0==waveForm)
		{
			LCD_PrintString("Sine");
		}
		else
		if (1==waveForm)
		{
			LCD_PrintString("Ramp");
		}
	}
	LCD_Position(1,3);
	CyDelay(10);	
}

uint8 freq_fast=0;
uint8 selected=0;
int32 last_freq=-1;

void do_ui_wavefreq(bool top_pressed, bool bottom_pressed, uint8 slider, bool init)
{
	if (init)
	{
		LCD_ClearDisplay();
		LCD_IsReady();
		LCD_PrintString("  freq   vco src");
		LCD_IsReady();
		
		LCD_Position(0,6);
		if (0==freq_fast)
		{
			LCD_PutChar('-');
		}
		else
		{
			LCD_PutChar('+');
		}

		LCD_Position(1,9);
		if (0==selectMode)
			LCD_PrintString("user   ");
		if (1==selectMode)
			LCD_PrintString("poti   ");
		if (2==selectMode)
			LCD_PrintString("ext vco");
		if (3==selectMode)
			LCD_PrintString("ext+pot");
		if (4==selectMode)
			LCD_PrintString("all    ");
		LCD_IsReady();

		selected=0;
		LCD_Position(1,1);
		LCD_PutChar(0);
		LCD_IsReady();
		LCD_Position(1,1);
	}

	if (bottom_pressed)
	{
		selected=1-selected;
		if (0==selected)
		{
			LCD_Position(1,8);
			LCD_PutChar(' ');
			LCD_IsReady();
			LCD_Position(1,1);
			LCD_PutChar(0);
			LCD_IsReady();
			LCD_Position(1,1);
		}
		else
		{
			LCD_Position(1,1);
			LCD_PutChar(' ');
			LCD_IsReady();
			LCD_Position(1,8);
			LCD_PutChar(0);
			LCD_IsReady();
			LCD_Position(1,8);
		}
	}
	if (0==selected)
	{
		// toggle fast mode
		if (top_pressed)
		{
			freq_fast=1-freq_fast;
			LCD_Position(0,6);
			if (0==freq_fast)
			{
				LCD_PutChar('-');
			}
			else
			{
				LCD_PutChar('+');
			}
		}

		// does the user want to change the frequency?
		if (0xff!=slider)
		{
			int16 rawPos2=(int16)slider;
			int8 pos=rawPos2-50; // pos is now -50..50

			// use slow mode?
			if (0==freq_fast)
				pos=pos/10;

			userFreq=userFreq+pos;
			if (userFreq<100)
				userFreq=100;
		}
	}
	else
	{
		if (top_pressed)
		{
			selectMode=(selectMode+1)%5;
			LCD_Position(1,9);
			if (0==selectMode)
				LCD_PrintString("user   ");
			if (1==selectMode)
				LCD_PrintString("poti   ");
			if (2==selectMode)
				LCD_PrintString("ext vco");
			if (3==selectMode)
				LCD_PrintString("ext+pot");
			if (4==selectMode)
				LCD_PrintString("all    ");
			LCD_IsReady();
		}
	}

	// now set the proper frequency
	int32 freq=0;

	if (0==selectMode || 4==selectMode)
	{
		freq+=userFreq;
	}
	if (1==selectMode || 3==selectMode || 4==selectMode)
	{
		freq+=(ADC_Freq_Samples[2]-VCO_OFFSET);
	}
	if (2==selectMode || 3==selectMode || 4==selectMode)
	{
		freq+=(ADC_Freq_Samples[0]-VCO_OFFSET);
		freq+=(ADC_Freq_Samples[1]-VCO_OFFSET);
	}
	if (0>freq)
		freq=0;
	if (freq<MIN_FREQ)
		freq=MIN_FREQ;

	setFrequency(freq);
	
	if (freq!=last_freq)
	{
		LCD_Position(1,2);
		LCD_PrintString("      ");
		LCD_IsReady();
		LCD_Position(1,2);
		LCD_PrintNumber(freq);
		LCD_IsReady();
		last_freq=freq;
	}
	
	if (0==selected)
	{
		LCD_Position(1,1);
	}
	else
	{
		LCD_Position(1,8);
	}
	
	CyDelay(10);	
}

/* [] END OF FILE */
