/* ========================================
 *
 * Copyright Hendrik Lipka, 2011
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF Hendrik Lipka.
 *
 * ========================================
*/

#include <device.h>
#include <stdbool.h>

#include "admscope.h"
#include "dma.h"
#include "callback.h"

#include "scope.h"

/* 
	parsed config values
*/
// analog trigger and input config data for both channels
uint8 triggerLevel1=0;
uint8 triggerRising1=0;
uint8 triggerActive1=0;
uint8 channelMux1=0;
uint8 channelGain1=1;

uint8 triggerLevel2=0;
uint8 triggerRising2=0;
uint8 triggerActive2=0;
uint8 channelGain2=1;
uint8 channelMux2=0;

// digital trigger data
uint8 triggerMaskDig=0;
uint8 triggerValuesDig=0;

// divider for frequency definition
uint32 divider=0;

// which channels are active
uint8 sendChannel0=1;
uint8 sendChannel1=1;
uint8 sendChannel2=1;
uint8 sendChannel3=1;

// number of samples to get
uint32 sampleCount=SAMPLE_BLOCK_SIZE;

void init_scope()
{
	// disable all triggers
	Reg_TriggerConfig_Write(0); 

	// start analog sub system
// disable first OpAmp, since otherwise we cannout place the wave output opamp
//	Opamp_1_Start();
	Comp_1_Start();
	PGA_1_Start();
	VDAC8_1_Start();
	AMux1_Start();
	AMux1_Select(0);
	
// disable second OpAmp, since otherwise we cannout place the wave output opamp
//	Opamp_2_Start();
	Comp_2_Start();
	PGA_2_Start();
	VDAC8_2_Start();
	AMux2_Start();
	AMux2_Select(0);

	ADC_SAR_1_Start();
	ADC_SAR_2_Start();
	ADC_SAR_1_StartConvert();
	ADC_SAR_2_StartConvert();

	Clock_ADC_Start();
	
	// set analog trigger level to 2v
	VDAC8_1_SetValue(128);
	VDAC8_2_SetValue(128);
}



uint8 calculate_gain(uint8 value)
{

	switch(value)
	{
		case 0:
			return PGA_1_PGA_GAIN_01;
		case 1:
			return PGA_1_PGA_GAIN_02;
		case 2:
			return PGA_1_PGA_GAIN_04;
		case 3:
			return PGA_1_PGA_GAIN_08;
		case 4:
			return PGA_1_PGA_GAIN_16;
		case 5:
			return PGA_1_PGA_GAIN_24;
		case 6:
			return PGA_1_PGA_GAIN_25;
		case 7:
			return PGA_1_PGA_GAIN_32;
		case 8:
			return PGA_1_PGA_GAIN_48;
		case 9:
			return PGA_1_PGA_GAIN_50;
		default:
			return PGA_1_PGA_GAIN_01;
	}
    
    return 0;
}

void startTrigger()
{
	// configure analog triggers
	AMux1_Select(channelMux1);
	AMux2_Select(channelMux2);
	
	VDAC8_1_SetValue(triggerLevel1);
	VDAC8_2_SetValue(triggerLevel2);
	
	PGA_1_SetGain(calculate_gain(channelGain1));
	PGA_2_SetGain(calculate_gain(channelGain2));
	
	uint8 triggerActive=((0==triggerActive1)&&(0==triggerActive2)&&(0==triggerMaskDig));
	
	uint8 trig=triggerActive1;
	trig+=triggerRising1*2;
	trig+=triggerActive2*4;
	trig+=triggerRising2*8;
	trig+=triggerActive?16:0; // if no other triggers are active, trigger with bus count
	trig+=(triggerMaskDig>0)?32:0; // trigger digital channel only if at least one bit is set in the mask
	
	// configure digital trigger
	Reg_DigitalValues_Write(triggerValuesDig);
	Reg_DigitalMask_Write(triggerMaskDig);
	
	// enable triggers
	Reg_TriggerConfig_Write(trig);

	// configure clock
	// calculate desired sample rate
	// OLS base clock is 100MHz, so the divider is based in that value
	uint32 rate = ( int )( ( 1.0 * 100000000 ) / ( divider + 1 ) );  
	
	// now calculate the needed period for the counter
	// since the internal clock for the ADCs runs with twice the frequency
	// we first need to double the divider, and then subtract 1 (the one is the clock period for reset the counter)
	// (so a period of 1 divides by 2, a period of 3 divides by 4)
	if (0==rate)
		rate=1;
	uint32 counter_period=((int)(1.0*500000/rate))*(MAIN_CLOCK/19); // base clock must be multiple of 9.5MHz (for 500kSpS we need 9.5MHz), we just calculate with the doubles vaulues

	// counter is 16 bit, meaning we cannot go lower than 15Hz (20Hz as send by the OLS client)
	if (counter_period>65535)
		counter_period=65535;
	if (0==counter_period)
		counter_period=1;
	Clock_ADC_Stop();
	Clock_ADC_SetDivider((uint16)counter_period-1);
	Clock_ADC_Start();

	// configure DMA
	start_dma();
	// arm trigger
	// pulse the trigger reset high
	// trigger is armed when it goes low again
	run_trigger=0;
	Reg_TriggerReset_Write(1);
	CyDelayUs(1U);
	Reg_TriggerReset_Write(0);
}


/* [] END OF FILE */
